@@ -43,12 +43,17 @@ sub _handle_post ($self, $c) {
 sub _handle_regular_request ($self, $c, $data, $session_id) {
   return $c->render(json => {error => 'Missing session ID'}, status => 400) unless $session_id;
 
+  $c->res->headers->header('Mcp-Session-Id' => $session_id);
   return $c->render(data => '', status => 202)
     unless defined(my $result = $self->_handle($data, {session_id => $session_id, controller => $c}));
 
-  return $result->then(sub { $c->render(json => $_[0], status => 200) })
-    if blessed($result) && $result->isa('Mojo::Promise');
-  $c->render(json => $result, status => 200);
+  # Sync
+  return $c->render(json => $result, status => 200) if !blessed($result) || !$result->isa('Mojo::Promise');
+
+  # Async
+  $c->inactivity_timeout(0);
+  $c->write_sse;
+  $result->then(sub { $c->write_sse({text => to_json($_[0])})->finish });
 }
 
 1;
